#include "install.h"
#include <fs_attr.h>
#include <string.h>
#include <fs_index.h>
#include <Query.h>
#include <String.h>
#include <VolumeRoster.h>
#include <Path.h>
#include <stdio.h>


void install_init(dev_t device) {
	fs_create_index(device,"package",B_STRING_TYPE,0);
}

void install_init(BVolume *volume) {
	if (volume == NULL) {
		BVolumeRoster roster;
		BVolume cur;
		
		while (roster.GetNextVolume(&cur) == B_OK) {
			if (!cur.KnowsQuery())
				continue;
			install_init(&cur);
		}
	} else
		install_init(volume->Device());
}

void mark_file(BNode *node_to_mark,const char *package_name) {
	attr_info info;
	if (node_to_mark->GetAttrInfo("package",&info) == B_OK) {
		char *old_package = new char[info.size + strlen(package_name) + 4];
		node_to_mark->ReadAttr("package",B_STRING_TYPE,0,old_package,info.size);
		strcat(old_package,", ");
		strcat(old_package,package_name);
		node_to_mark->WriteAttr("package",B_STRING_TYPE,0,old_package,strlen(old_package) + 1);
		delete [] old_package;
	} else
		node_to_mark->WriteAttr("package",B_STRING_TYPE,0,package_name,strlen(package_name) + 1);
}

void mark_dir(BDirectory *dir,const char *package_name) {
	mark_file(dir,package_name);
	BEntry cur;
	BNode node;
	BDirectory recurse;
	while (dir->GetNextEntry(&cur) == B_OK) {
		node.SetTo(&cur);
		mark_file(&node,package_name);
		if (cur.IsDirectory()) {
			recurse.SetTo(&cur);
			mark_dir(&recurse,package_name);
		}
	}
}

BEntryList *get_files(const char *package_name,BVolume *volume) {
	BQuery *query = new BQuery();
	BString predicate = "(package==\"*";
	predicate += package_name;
	predicate += "*\")";
	query->SetPredicate(predicate.String());
	query->SetVolume(volume);
	query->Fetch();
	return query;
}

int32 uninstall(const char *package_name) {
	BVolumeRoster roster;
	BVolume cur;
	int32 total = 0;
	
	while (roster.GetNextVolume(&cur) == B_OK) {
		if (!cur.KnowsQuery())
			continue;
		total += uninstall_from_volume(package_name,&cur);
	}
	return total;
}

int32 uninstall_from_volume(const char *package_name,BVolume *volume) { //---------Returns num files removed
	int32 files_removed = 0;
	
	
	BEntryList *files = get_files(package_name,volume);
	BEntry cur;
	BNode review;
	BList failures; //---Directories, come back to these
	BString package_list;
	BPath path;
	
	while (files->GetNextEntry(&cur) == B_OK) {
		 cur.GetPath(&path);
		 printf("\tRemoving %s\n",path.Path());
		 review.SetTo(&cur);
		 review.ReadAttrString("package",&package_list);
		 if (package_list != package_name) {
			BString test;
			test << ", " << package_name;
			package_list.RemoveFirst(test);
			test << package_name << ", ";
			package_list.RemoveFirst(test);
			review.WriteAttrString("package",&package_list);
			continue;
		}
		if (cur.Remove() != B_OK)
			failures.AddItem(new BEntry(cur));
		else
			files_removed++;
	}
	BEntry *dir;
	int32 a = failures.CountItems();
	for (int32 b = 0; b < a; b++) { //---Maybe some are inside others--
		for (int32 i = 0; (dir = (BEntry *)(failures.ItemAt(i))) != NULL; i++) {
			if (dir->Remove() == B_OK) {
				failures.RemoveItem(dir);
				files_removed++;
				delete dir;
				i--;
				continue;
			}
		}
		if (failures.CountItems() == 0)
			break; //---We're done
	}
	a = failures.CountItems();
	for (int i = 0; i < a; i ++) { //--Clean up
		delete failures.ItemAt(i);
	}
	return files_removed;
}